chai = require 'chai'
expect = chai.expect

ecc = require 'ecc-tools'

Script = require '../src/script'

p2pkh = '76a914c86a5e5deaeefc79ef444d40bb896f7c6253e8ab88ac'
p2pkh_asm = 'OP_DUP OP_HASH160 c86a5e5deaeefc79ef444d40bb896f7c6253e8ab OP_EQUALVERIFY OP_CHECKSIG'
p2pkh_codes = new Buffer('76a91488ac', 'hex')

p2pkhm = '76a91459288e0ec97813bab904867cdcae0d681f6cce5188ac1c73706b71ca325ab67a8f1decb61546a8950da859a08601000000000075'
p2pkhm_meta = '73706b71ca325ab67a8f1decb61546a8950da859a086010000000000'

private_key = ecc.sha256('hello world')
public_key = ecc.publicKey(private_key, true)
sighash = ecc.sha256(public_key)
signature = ecc.sign(ecc.reverse(sighash), private_key)

hash = ecc.sha256ripemd160(public_key)

describe 'Script', ->
  describe '#instance', ->
    it 'should return a Parser instance', ->
      script = Script()
      expect(script).to.be.instanceof(Script)

  describe 'opscodes', ->
    it 'should break a raw script into opscodes', ->
      script = Script(p2pkh)
      expect(script.opcodes.length).to.equal(5)

  describe 'asm', ->
    it 'should return a human readable version of the script', ->
      script = Script(p2pkh)
      expect(script.asm()).to.equal(p2pkh_asm)

  describe 'buf', ->
    it 'should return a human readable version of the script', ->
      script = Script(p2pkh)
      expect(script.buf()).to.eql(new Buffer(p2pkh, 'hex'))

  describe 'buf', ->
    it 'should return a human readable version of the script', ->
      script = Script(p2pkh)
      expect(script.hex()).to.eql(p2pkh)

  describe 'codes', ->
    it 'should return a buffer of codes', ->
      script = Script(p2pkh)
      expect(script.codes()).to.eql(p2pkh_codes)

  describe 'classify', ->
    it 'should detect a p2pkh script', ->
      script = Script(p2pkh)
      expect(script.classify()).to.eql('p2pkh')

    it 'should detect a p2pkhm script', ->
      script = Script(p2pkhm)
      expect(script.classify()).to.eql('p2pkhm')

  describe 'meta', ->
    it 'should return an array with classification and metadata', ->
      script = Script(p2pkhm)
      expect(script.meta()).to.eql(['p2pkhm', new Buffer(p2pkhm_meta, 'hex')])

  describe 'destination', ->
    it 'should return an array with public key hash for p2pkh variants', ->
      script = Script(p2pkhm)
      expect(script.destination()).to.eql(['59288e0ec97813bab904867cdcae0d681f6cce51'])

  describe 'p2pkh_sig', ->
    it 'should return signature script', ->
      script = Script.p2pkh_sig(private_key, sighash).buf()
      expect(script.toString('hex')).to.eql((signature.length + 1).toString(16) + signature.toString('hex') + '0121' + public_key.toString('hex'))

